# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .voice_response import VoiceResponse

__all__ = ["VoiceListResponse"]

VoiceListResponse: TypeAlias = List[VoiceResponse]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ..types import phone_number_create_params, phone_number_import_params, phone_number_update_params
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.phone_number_response import PhoneNumberResponse
from ..types.phone_number_list_response import PhoneNumberListResponse

__all__ = ["PhoneNumberResource", "AsyncPhoneNumberResource"]


class PhoneNumberResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PhoneNumberResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#accessing-raw-response-data-eg-headers
        """
        return PhoneNumberResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PhoneNumberResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#with_streaming_response
        """
        return PhoneNumberResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        area_code: int | Omit = omit,
        country_code: Literal["US", "CA"] | Omit = omit,
        inbound_agent_id: Optional[str] | Omit = omit,
        inbound_agent_version: Optional[int] | Omit = omit,
        inbound_webhook_url: Optional[str] | Omit = omit,
        nickname: str | Omit = omit,
        number_provider: Literal["twilio", "telnyx"] | Omit = omit,
        outbound_agent_id: Optional[str] | Omit = omit,
        outbound_agent_version: Optional[int] | Omit = omit,
        phone_number: str | Omit = omit,
        toll_free: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberResponse:
        """
        Buy a new phone number & Bind agents

        Args:
          area_code: Area code of the number to obtain. Format is a 3 digit integer. Currently only
              supports US area code.

          country_code: The ISO 3166-1 alpha-2 country code of the number you are trying to purchase. If
              left empty, will default to "US".

          inbound_agent_id: Unique id of agent to bind to the number. The number will automatically use the
              agent when receiving inbound calls. If null, this number would not accept
              inbound call.

          inbound_agent_version: Version of the inbound agent to bind to the number. If not provided, will
              default to latest version.

          inbound_webhook_url: If set, will send a webhook for inbound calls, where you can to override agent
              id, set dynamic variables and other fields specific to that call.

          nickname: Nickname of the number. This is for your reference only.

          number_provider: The provider to purchase the phone number from. Default to twilio.

          outbound_agent_id: Unique id of agent to bind to the number. The number will automatically use the
              agent when conducting outbound calls. If null, this number would not be able to
              initiate outbound call without agent id override.

          outbound_agent_version: Version of the outbound agent to bind to the number. If not provided, will
              default to latest version.

          phone_number: The number you are trying to purchase in E.164 format of the number (+country
              code then number with no space and no special characters).

          toll_free: Whether to purchase a toll-free number. Toll-free numbers incur higher costs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/create-phone-number",
            body=maybe_transform(
                {
                    "area_code": area_code,
                    "country_code": country_code,
                    "inbound_agent_id": inbound_agent_id,
                    "inbound_agent_version": inbound_agent_version,
                    "inbound_webhook_url": inbound_webhook_url,
                    "nickname": nickname,
                    "number_provider": number_provider,
                    "outbound_agent_id": outbound_agent_id,
                    "outbound_agent_version": outbound_agent_version,
                    "phone_number": phone_number,
                    "toll_free": toll_free,
                },
                phone_number_create_params.PhoneNumberCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberResponse,
        )

    def retrieve(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberResponse:
        """
        Retrieve details of a specific phone number

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return self._get(
            f"/get-phone-number/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberResponse,
        )

    def update(
        self,
        phone_number: str,
        *,
        inbound_agent_id: Optional[str] | Omit = omit,
        inbound_agent_version: Optional[int] | Omit = omit,
        inbound_webhook_url: Optional[str] | Omit = omit,
        nickname: Optional[str] | Omit = omit,
        outbound_agent_id: Optional[str] | Omit = omit,
        outbound_agent_version: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberResponse:
        """
        Update agent bound to a purchased phone number

        Args:
          inbound_agent_id: Unique id of agent to bind to the number. The number will automatically use the
              agent when receiving inbound calls. If set to null, this number would not accept
              inbound call.

          inbound_agent_version: Version of the inbound agent to bind to the number. If not provided, will
              default to latest version.

          inbound_webhook_url: If set, will send a webhook for inbound calls, where you can to override agent
              id, set dynamic variables and other fields specific to that call.

          nickname: Nickname of the number. This is for your reference only.

          outbound_agent_id: Unique id of agent to bind to the number. The number will automatically use the
              agent when conducting outbound calls. If set to null, this number would not be
              able to initiate outbound call without agent id override.

          outbound_agent_version: Version of the outbound agent to bind to the number. If not provided, will
              default to latest version.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return self._patch(
            f"/update-phone-number/{phone_number}",
            body=maybe_transform(
                {
                    "inbound_agent_id": inbound_agent_id,
                    "inbound_agent_version": inbound_agent_version,
                    "inbound_webhook_url": inbound_webhook_url,
                    "nickname": nickname,
                    "outbound_agent_id": outbound_agent_id,
                    "outbound_agent_version": outbound_agent_version,
                },
                phone_number_update_params.PhoneNumberUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberListResponse:
        """List all phone numbers"""
        return self._get(
            "/list-phone-numbers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberListResponse,
        )

    def delete(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an existing phone number

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/delete-phone-number/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def import_(
        self,
        *,
        phone_number: str,
        termination_uri: str,
        inbound_agent_id: Optional[str] | Omit = omit,
        inbound_agent_version: Optional[int] | Omit = omit,
        inbound_webhook_url: Optional[str] | Omit = omit,
        nickname: str | Omit = omit,
        outbound_agent_id: str | Omit = omit,
        outbound_agent_version: Optional[int] | Omit = omit,
        sip_trunk_auth_password: str | Omit = omit,
        sip_trunk_auth_username: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberResponse:
        """
        Import a phone number from custom telephony & Bind agents

        Args:
          phone_number: The number you are trying to import in E.164 format of the number (+country
              code, then number with no space, no special characters), used as the unique
              identifier for phone number APIs.

          termination_uri: The termination uri to uniquely identify your elastic SIP trunk. This is used
              for outbound calls. For Twilio elastic SIP trunks it always end with
              ".pstn.twilio.com".

          inbound_agent_id: Unique id of agent to bind to the number. The number will automatically use the
              agent when receiving inbound calls. If null, this number would not accept
              inbound call.

          inbound_agent_version: Version of the inbound agent to bind to the number. If not provided, will
              default to latest version.

          inbound_webhook_url: If set, will send a webhook for inbound calls, where you can to override agent
              id, set dynamic variables and other fields specific to that call.

          nickname: Nickname of the number. This is for your reference only.

          outbound_agent_id: Unique id of agent to bind to the number. The number will automatically use the
              agent when conducting outbound calls. If null, this number would not be able to
              initiate outbound call without agent id override.

          outbound_agent_version: Version of the outbound agent to bind to the number. If not provided, will
              default to latest version.

          sip_trunk_auth_password: The password used for authentication for the SIP trunk.

          sip_trunk_auth_username: The username used for authentication for the SIP trunk.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/import-phone-number",
            body=maybe_transform(
                {
                    "phone_number": phone_number,
                    "termination_uri": termination_uri,
                    "inbound_agent_id": inbound_agent_id,
                    "inbound_agent_version": inbound_agent_version,
                    "inbound_webhook_url": inbound_webhook_url,
                    "nickname": nickname,
                    "outbound_agent_id": outbound_agent_id,
                    "outbound_agent_version": outbound_agent_version,
                    "sip_trunk_auth_password": sip_trunk_auth_password,
                    "sip_trunk_auth_username": sip_trunk_auth_username,
                },
                phone_number_import_params.PhoneNumberImportParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberResponse,
        )


class AsyncPhoneNumberResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPhoneNumberResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncPhoneNumberResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPhoneNumberResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#with_streaming_response
        """
        return AsyncPhoneNumberResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        area_code: int | Omit = omit,
        country_code: Literal["US", "CA"] | Omit = omit,
        inbound_agent_id: Optional[str] | Omit = omit,
        inbound_agent_version: Optional[int] | Omit = omit,
        inbound_webhook_url: Optional[str] | Omit = omit,
        nickname: str | Omit = omit,
        number_provider: Literal["twilio", "telnyx"] | Omit = omit,
        outbound_agent_id: Optional[str] | Omit = omit,
        outbound_agent_version: Optional[int] | Omit = omit,
        phone_number: str | Omit = omit,
        toll_free: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberResponse:
        """
        Buy a new phone number & Bind agents

        Args:
          area_code: Area code of the number to obtain. Format is a 3 digit integer. Currently only
              supports US area code.

          country_code: The ISO 3166-1 alpha-2 country code of the number you are trying to purchase. If
              left empty, will default to "US".

          inbound_agent_id: Unique id of agent to bind to the number. The number will automatically use the
              agent when receiving inbound calls. If null, this number would not accept
              inbound call.

          inbound_agent_version: Version of the inbound agent to bind to the number. If not provided, will
              default to latest version.

          inbound_webhook_url: If set, will send a webhook for inbound calls, where you can to override agent
              id, set dynamic variables and other fields specific to that call.

          nickname: Nickname of the number. This is for your reference only.

          number_provider: The provider to purchase the phone number from. Default to twilio.

          outbound_agent_id: Unique id of agent to bind to the number. The number will automatically use the
              agent when conducting outbound calls. If null, this number would not be able to
              initiate outbound call without agent id override.

          outbound_agent_version: Version of the outbound agent to bind to the number. If not provided, will
              default to latest version.

          phone_number: The number you are trying to purchase in E.164 format of the number (+country
              code then number with no space and no special characters).

          toll_free: Whether to purchase a toll-free number. Toll-free numbers incur higher costs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/create-phone-number",
            body=await async_maybe_transform(
                {
                    "area_code": area_code,
                    "country_code": country_code,
                    "inbound_agent_id": inbound_agent_id,
                    "inbound_agent_version": inbound_agent_version,
                    "inbound_webhook_url": inbound_webhook_url,
                    "nickname": nickname,
                    "number_provider": number_provider,
                    "outbound_agent_id": outbound_agent_id,
                    "outbound_agent_version": outbound_agent_version,
                    "phone_number": phone_number,
                    "toll_free": toll_free,
                },
                phone_number_create_params.PhoneNumberCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberResponse,
        )

    async def retrieve(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberResponse:
        """
        Retrieve details of a specific phone number

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return await self._get(
            f"/get-phone-number/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberResponse,
        )

    async def update(
        self,
        phone_number: str,
        *,
        inbound_agent_id: Optional[str] | Omit = omit,
        inbound_agent_version: Optional[int] | Omit = omit,
        inbound_webhook_url: Optional[str] | Omit = omit,
        nickname: Optional[str] | Omit = omit,
        outbound_agent_id: Optional[str] | Omit = omit,
        outbound_agent_version: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberResponse:
        """
        Update agent bound to a purchased phone number

        Args:
          inbound_agent_id: Unique id of agent to bind to the number. The number will automatically use the
              agent when receiving inbound calls. If set to null, this number would not accept
              inbound call.

          inbound_agent_version: Version of the inbound agent to bind to the number. If not provided, will
              default to latest version.

          inbound_webhook_url: If set, will send a webhook for inbound calls, where you can to override agent
              id, set dynamic variables and other fields specific to that call.

          nickname: Nickname of the number. This is for your reference only.

          outbound_agent_id: Unique id of agent to bind to the number. The number will automatically use the
              agent when conducting outbound calls. If set to null, this number would not be
              able to initiate outbound call without agent id override.

          outbound_agent_version: Version of the outbound agent to bind to the number. If not provided, will
              default to latest version.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return await self._patch(
            f"/update-phone-number/{phone_number}",
            body=await async_maybe_transform(
                {
                    "inbound_agent_id": inbound_agent_id,
                    "inbound_agent_version": inbound_agent_version,
                    "inbound_webhook_url": inbound_webhook_url,
                    "nickname": nickname,
                    "outbound_agent_id": outbound_agent_id,
                    "outbound_agent_version": outbound_agent_version,
                },
                phone_number_update_params.PhoneNumberUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberListResponse:
        """List all phone numbers"""
        return await self._get(
            "/list-phone-numbers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberListResponse,
        )

    async def delete(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an existing phone number

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/delete-phone-number/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def import_(
        self,
        *,
        phone_number: str,
        termination_uri: str,
        inbound_agent_id: Optional[str] | Omit = omit,
        inbound_agent_version: Optional[int] | Omit = omit,
        inbound_webhook_url: Optional[str] | Omit = omit,
        nickname: str | Omit = omit,
        outbound_agent_id: str | Omit = omit,
        outbound_agent_version: Optional[int] | Omit = omit,
        sip_trunk_auth_password: str | Omit = omit,
        sip_trunk_auth_username: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberResponse:
        """
        Import a phone number from custom telephony & Bind agents

        Args:
          phone_number: The number you are trying to import in E.164 format of the number (+country
              code, then number with no space, no special characters), used as the unique
              identifier for phone number APIs.

          termination_uri: The termination uri to uniquely identify your elastic SIP trunk. This is used
              for outbound calls. For Twilio elastic SIP trunks it always end with
              ".pstn.twilio.com".

          inbound_agent_id: Unique id of agent to bind to the number. The number will automatically use the
              agent when receiving inbound calls. If null, this number would not accept
              inbound call.

          inbound_agent_version: Version of the inbound agent to bind to the number. If not provided, will
              default to latest version.

          inbound_webhook_url: If set, will send a webhook for inbound calls, where you can to override agent
              id, set dynamic variables and other fields specific to that call.

          nickname: Nickname of the number. This is for your reference only.

          outbound_agent_id: Unique id of agent to bind to the number. The number will automatically use the
              agent when conducting outbound calls. If null, this number would not be able to
              initiate outbound call without agent id override.

          outbound_agent_version: Version of the outbound agent to bind to the number. If not provided, will
              default to latest version.

          sip_trunk_auth_password: The password used for authentication for the SIP trunk.

          sip_trunk_auth_username: The username used for authentication for the SIP trunk.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/import-phone-number",
            body=await async_maybe_transform(
                {
                    "phone_number": phone_number,
                    "termination_uri": termination_uri,
                    "inbound_agent_id": inbound_agent_id,
                    "inbound_agent_version": inbound_agent_version,
                    "inbound_webhook_url": inbound_webhook_url,
                    "nickname": nickname,
                    "outbound_agent_id": outbound_agent_id,
                    "outbound_agent_version": outbound_agent_version,
                    "sip_trunk_auth_password": sip_trunk_auth_password,
                    "sip_trunk_auth_username": sip_trunk_auth_username,
                },
                phone_number_import_params.PhoneNumberImportParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberResponse,
        )


class PhoneNumberResourceWithRawResponse:
    def __init__(self, phone_number: PhoneNumberResource) -> None:
        self._phone_number = phone_number

        self.create = to_raw_response_wrapper(
            phone_number.create,
        )
        self.retrieve = to_raw_response_wrapper(
            phone_number.retrieve,
        )
        self.update = to_raw_response_wrapper(
            phone_number.update,
        )
        self.list = to_raw_response_wrapper(
            phone_number.list,
        )
        self.delete = to_raw_response_wrapper(
            phone_number.delete,
        )
        self.import_ = to_raw_response_wrapper(
            phone_number.import_,
        )


class AsyncPhoneNumberResourceWithRawResponse:
    def __init__(self, phone_number: AsyncPhoneNumberResource) -> None:
        self._phone_number = phone_number

        self.create = async_to_raw_response_wrapper(
            phone_number.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            phone_number.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            phone_number.update,
        )
        self.list = async_to_raw_response_wrapper(
            phone_number.list,
        )
        self.delete = async_to_raw_response_wrapper(
            phone_number.delete,
        )
        self.import_ = async_to_raw_response_wrapper(
            phone_number.import_,
        )


class PhoneNumberResourceWithStreamingResponse:
    def __init__(self, phone_number: PhoneNumberResource) -> None:
        self._phone_number = phone_number

        self.create = to_streamed_response_wrapper(
            phone_number.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            phone_number.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            phone_number.update,
        )
        self.list = to_streamed_response_wrapper(
            phone_number.list,
        )
        self.delete = to_streamed_response_wrapper(
            phone_number.delete,
        )
        self.import_ = to_streamed_response_wrapper(
            phone_number.import_,
        )


class AsyncPhoneNumberResourceWithStreamingResponse:
    def __init__(self, phone_number: AsyncPhoneNumberResource) -> None:
        self._phone_number = phone_number

        self.create = async_to_streamed_response_wrapper(
            phone_number.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            phone_number.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            phone_number.update,
        )
        self.list = async_to_streamed_response_wrapper(
            phone_number.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            phone_number.delete,
        )
        self.import_ = async_to_streamed_response_wrapper(
            phone_number.import_,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .chat_response import ChatResponse

__all__ = ["ChatListResponse"]

ChatListResponse: TypeAlias = List[ChatResponse]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from retell import Retell, AsyncRetell
from tests.utils import assert_matches_type
from retell.types import (
    CallResponse,
    WebCallResponse,
    CallListResponse,
    PhoneCallResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCall:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: Retell) -> None:
        call = client.call.retrieve(
            "119c3f8e47135a29e65947eeb34cf12d",
        )
        assert_matches_type(CallResponse, call, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Retell) -> None:
        response = client.call.with_raw_response.retrieve(
            "119c3f8e47135a29e65947eeb34cf12d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call = response.parse()
        assert_matches_type(CallResponse, call, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Retell) -> None:
        with client.call.with_streaming_response.retrieve(
            "119c3f8e47135a29e65947eeb34cf12d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call = response.parse()
            assert_matches_type(CallResponse, call, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `call_id` but received ''"):
            client.call.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: Retell) -> None:
        call = client.call.update(
            call_id="call_a4441234567890777c4a4a123e6",
        )
        assert_matches_type(CallResponse, call, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: Retell) -> None:
        call = client.call.update(
            call_id="call_a4441234567890777c4a4a123e6",
            data_storage_setting="everything_except_pii",
            metadata={
                "customer_id": "cust_123",
                "notes": "Follow-up required",
            },
            override_dynamic_variables={"additional_discount": "15%"},
        )
        assert_matches_type(CallResponse, call, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Retell) -> None:
        response = client.call.with_raw_response.update(
            call_id="call_a4441234567890777c4a4a123e6",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call = response.parse()
        assert_matches_type(CallResponse, call, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Retell) -> None:
        with client.call.with_streaming_response.update(
            call_id="call_a4441234567890777c4a4a123e6",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call = response.parse()
            assert_matches_type(CallResponse, call, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `call_id` but received ''"):
            client.call.with_raw_response.update(
                call_id="",
            )

    @parametrize
    def test_method_list(self, client: Retell) -> None:
        call = client.call.list()
        assert_matches_type(CallListResponse, call, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Retell) -> None:
        call = client.call.list(
            filter_criteria={
                "agent_id": ["agent_oBeDLoLOeuAbiuaMFXRtDOLriT12345"],
                "batch_call_id": ["string"],
                "call_status": ["ended"],
                "call_successful": [True],
                "call_type": ["phone_call"],
                "direction": ["inbound"],
                "disconnection_reason": ["user_hangup"],
                "duration_ms": {
                    "lower_threshold": 0,
                    "upper_threshold": 0,
                },
                "e2e_latency_p50": {
                    "lower_threshold": 0,
                    "upper_threshold": 0,
                },
                "from_number": ["string"],
                "in_voicemail": [True],
                "start_timestamp": {
                    "lower_threshold": 1738475411000,
                    "upper_threshold": 1738475421000,
                },
                "to_number": ["string"],
                "user_sentiment": ["Positive"],
                "version": [0],
            },
            limit=0,
            pagination_key="pagination_key",
            sort_order="ascending",
        )
        assert_matches_type(CallListResponse, call, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Retell) -> None:
        response = client.call.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call = response.parse()
        assert_matches_type(CallListResponse, call, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Retell) -> None:
        with client.call.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call = response.parse()
            assert_matches_type(CallListResponse, call, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Retell) -> None:
        call = client.call.delete(
            "119c3f8e47135a29e65947eeb34cf12d",
        )
        assert call is None

    @parametrize
    def test_raw_response_delete(self, client: Retell) -> None:
        response = client.call.with_raw_response.delete(
            "119c3f8e47135a29e65947eeb34cf12d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call = response.parse()
        assert call is None

    @parametrize
    def test_streaming_response_delete(self, client: Retell) -> None:
        with client.call.with_streaming_response.delete(
            "119c3f8e47135a29e65947eeb34cf12d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call = response.parse()
            assert call is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `call_id` but received ''"):
            client.call.with_raw_response.delete(
                "",
            )

    @parametrize
    def test_method_create_phone_call(self, client: Retell) -> None:
        call = client.call.create_phone_call(
            from_number="+14157774444",
            to_number="+12137774445",
        )
        assert_matches_type(PhoneCallResponse, call, path=["response"])

    @parametrize
    def test_method_create_phone_call_with_all_params(self, client: Retell) -> None:
        call = client.call.create_phone_call(
            from_number="+14157774444",
            to_number="+12137774445",
            agent_override={
                "agent": {
                    "agent_name": "Jarvis",
                    "allow_user_dtmf": True,
                    "ambient_sound": "coffee-shop",
                    "ambient_sound_volume": 1,
                    "backchannel_frequency": 0.9,
                    "backchannel_words": ["yeah", "uh-huh"],
                    "begin_message_delay_ms": 1000,
                    "boosted_keywords": ["retell", "kroger"],
                    "data_storage_setting": "everything",
                    "denoising_mode": "noise-cancellation",
                    "enable_backchannel": True,
                    "end_call_after_silence_ms": 600000,
                    "fallback_voice_ids": ["openai-Alloy", "deepgram-Angus"],
                    "interruption_sensitivity": 1,
                    "language": "en-US",
                    "max_call_duration_ms": 3600000,
                    "normalize_for_speech": True,
                    "opt_in_signed_url": True,
                    "pii_config": {
                        "categories": ["person_name"],
                        "mode": "post_call",
                    },
                    "post_call_analysis_data": [
                        {
                            "description": "The name of the customer.",
                            "name": "customer_name",
                            "type": "string",
                            "examples": ["John Doe", "Jane Smith"],
                        }
                    ],
                    "post_call_analysis_model": "gpt-4o-mini",
                    "pronunciation_dictionary": [
                        {
                            "alphabet": "ipa",
                            "phoneme": "ˈæktʃuəli",
                            "word": "actually",
                        }
                    ],
                    "reminder_max_count": 2,
                    "reminder_trigger_ms": 10000,
                    "response_engine": {
                        "llm_id": "llm_234sdertfsdsfsdf",
                        "type": "retell-llm",
                        "version": 0,
                    },
                    "responsiveness": 1,
                    "ring_duration_ms": 30000,
                    "stt_mode": "fast",
                    "user_dtmf_options": {
                        "digit_limit": 1,
                        "termination_key": "#",
                        "timeout_ms": 1000,
                    },
                    "vocab_specialization": "general",
                    "voice_id": "11labs-Adrian",
                    "voice_model": "eleven_turbo_v2",
                    "voice_speed": 1,
                    "voice_temperature": 1,
                    "voicemail_option": {
                        "action": {
                            "text": "Please give us a callback tomorrow at 10am.",
                            "type": "static_text",
                        }
                    },
                    "volume": 1,
                    "webhook_timeout_ms": 10000,
                    "webhook_url": "https://webhook-url-here",
                },
                "conversation_flow": {
                    "begin_after_user_silence_ms": 2000,
                    "kb_config": {
                        "filter_score": 0.6,
                        "top_k": 3,
                    },
                    "knowledge_base_ids": ["kb_001", "kb_002"],
                    "model_choice": {
                        "model": "gpt-5",
                        "type": "cascading",
                        "high_priority": True,
                    },
                    "model_temperature": 0.7,
                    "start_speaker": "agent",
                    "tool_call_strict_mode": True,
                },
                "retell_llm": {
                    "begin_after_user_silence_ms": 2000,
                    "begin_message": "Hey I am a virtual assistant calling from Retell Hospital.",
                    "kb_config": {
                        "filter_score": 0.6,
                        "top_k": 3,
                    },
                    "knowledge_base_ids": ["string"],
                    "model": "gpt-4.1",
                    "model_high_priority": True,
                    "model_temperature": 0,
                    "s2s_model": "gpt-4o-realtime",
                    "start_speaker": "user",
                    "tool_call_strict_mode": True,
                },
            },
            custom_sip_headers={"X-Custom-Header": "Custom Value"},
            ignore_e164_validation=True,
            metadata={},
            override_agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
            override_agent_version=1,
            retell_llm_dynamic_variables={"customer_name": "bar"},
        )
        assert_matches_type(PhoneCallResponse, call, path=["response"])

    @parametrize
    def test_raw_response_create_phone_call(self, client: Retell) -> None:
        response = client.call.with_raw_response.create_phone_call(
            from_number="+14157774444",
            to_number="+12137774445",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call = response.parse()
        assert_matches_type(PhoneCallResponse, call, path=["response"])

    @parametrize
    def test_streaming_response_create_phone_call(self, client: Retell) -> None:
        with client.call.with_streaming_response.create_phone_call(
            from_number="+14157774444",
            to_number="+12137774445",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call = response.parse()
            assert_matches_type(PhoneCallResponse, call, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_create_web_call(self, client: Retell) -> None:
        call = client.call.create_web_call(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        )
        assert_matches_type(WebCallResponse, call, path=["response"])

    @parametrize
    def test_method_create_web_call_with_all_params(self, client: Retell) -> None:
        call = client.call.create_web_call(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
            agent_override={
                "agent": {
                    "agent_name": "Jarvis",
                    "allow_user_dtmf": True,
                    "ambient_sound": "coffee-shop",
                    "ambient_sound_volume": 1,
                    "backchannel_frequency": 0.9,
                    "backchannel_words": ["yeah", "uh-huh"],
                    "begin_message_delay_ms": 1000,
                    "boosted_keywords": ["retell", "kroger"],
                    "data_storage_setting": "everything",
                    "denoising_mode": "noise-cancellation",
                    "enable_backchannel": True,
                    "end_call_after_silence_ms": 600000,
                    "fallback_voice_ids": ["openai-Alloy", "deepgram-Angus"],
                    "interruption_sensitivity": 1,
                    "language": "en-US",
                    "max_call_duration_ms": 3600000,
                    "normalize_for_speech": True,
                    "opt_in_signed_url": True,
                    "pii_config": {
                        "categories": ["person_name"],
                        "mode": "post_call",
                    },
                    "post_call_analysis_data": [
                        {
                            "description": "The name of the customer.",
                            "name": "customer_name",
                            "type": "string",
                            "examples": ["John Doe", "Jane Smith"],
                        }
                    ],
                    "post_call_analysis_model": "gpt-4o-mini",
                    "pronunciation_dictionary": [
                        {
                            "alphabet": "ipa",
                            "phoneme": "ˈæktʃuəli",
                            "word": "actually",
                        }
                    ],
                    "reminder_max_count": 2,
                    "reminder_trigger_ms": 10000,
                    "response_engine": {
                        "llm_id": "llm_234sdertfsdsfsdf",
                        "type": "retell-llm",
                        "version": 0,
                    },
                    "responsiveness": 1,
                    "ring_duration_ms": 30000,
                    "stt_mode": "fast",
                    "user_dtmf_options": {
                        "digit_limit": 1,
                        "termination_key": "#",
                        "timeout_ms": 1000,
                    },
                    "vocab_specialization": "general",
                    "voice_id": "11labs-Adrian",
                    "voice_model": "eleven_turbo_v2",
                    "voice_speed": 1,
                    "voice_temperature": 1,
                    "voicemail_option": {
                        "action": {
                            "text": "Please give us a callback tomorrow at 10am.",
                            "type": "static_text",
                        }
                    },
                    "volume": 1,
                    "webhook_timeout_ms": 10000,
                    "webhook_url": "https://webhook-url-here",
                },
                "conversation_flow": {
                    "begin_after_user_silence_ms": 2000,
                    "kb_config": {
                        "filter_score": 0.6,
                        "top_k": 3,
                    },
                    "knowledge_base_ids": ["kb_001", "kb_002"],
                    "model_choice": {
                        "model": "gpt-5",
                        "type": "cascading",
                        "high_priority": True,
                    },
                    "model_temperature": 0.7,
                    "start_speaker": "agent",
                    "tool_call_strict_mode": True,
                },
                "retell_llm": {
                    "begin_after_user_silence_ms": 2000,
                    "begin_message": "Hey I am a virtual assistant calling from Retell Hospital.",
                    "kb_config": {
                        "filter_score": 0.6,
                        "top_k": 3,
                    },
                    "knowledge_base_ids": ["string"],
                    "model": "gpt-4.1",
                    "model_high_priority": True,
                    "model_temperature": 0,
                    "s2s_model": "gpt-4o-realtime",
                    "start_speaker": "user",
                    "tool_call_strict_mode": True,
                },
            },
            agent_version=1,
            metadata={},
            retell_llm_dynamic_variables={"customer_name": "bar"},
        )
        assert_matches_type(WebCallResponse, call, path=["response"])

    @parametrize
    def test_raw_response_create_web_call(self, client: Retell) -> None:
        response = client.call.with_raw_response.create_web_call(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call = response.parse()
        assert_matches_type(WebCallResponse, call, path=["response"])

    @parametrize
    def test_streaming_response_create_web_call(self, client: Retell) -> None:
        with client.call.with_streaming_response.create_web_call(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call = response.parse()
            assert_matches_type(WebCallResponse, call, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_register_phone_call(self, client: Retell) -> None:
        call = client.call.register_phone_call(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        )
        assert_matches_type(PhoneCallResponse, call, path=["response"])

    @parametrize
    def test_method_register_phone_call_with_all_params(self, client: Retell) -> None:
        call = client.call.register_phone_call(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
            agent_override={
                "agent": {
                    "agent_name": "Jarvis",
                    "allow_user_dtmf": True,
                    "ambient_sound": "coffee-shop",
                    "ambient_sound_volume": 1,
                    "backchannel_frequency": 0.9,
                    "backchannel_words": ["yeah", "uh-huh"],
                    "begin_message_delay_ms": 1000,
                    "boosted_keywords": ["retell", "kroger"],
                    "data_storage_setting": "everything",
                    "denoising_mode": "noise-cancellation",
                    "enable_backchannel": True,
                    "end_call_after_silence_ms": 600000,
                    "fallback_voice_ids": ["openai-Alloy", "deepgram-Angus"],
                    "interruption_sensitivity": 1,
                    "language": "en-US",
                    "max_call_duration_ms": 3600000,
                    "normalize_for_speech": True,
                    "opt_in_signed_url": True,
                    "pii_config": {
                        "categories": ["person_name"],
                        "mode": "post_call",
                    },
                    "post_call_analysis_data": [
                        {
                            "description": "The name of the customer.",
                            "name": "customer_name",
                            "type": "string",
                            "examples": ["John Doe", "Jane Smith"],
                        }
                    ],
                    "post_call_analysis_model": "gpt-4o-mini",
                    "pronunciation_dictionary": [
                        {
                            "alphabet": "ipa",
                            "phoneme": "ˈæktʃuəli",
                            "word": "actually",
                        }
                    ],
                    "reminder_max_count": 2,
                    "reminder_trigger_ms": 10000,
                    "response_engine": {
                        "llm_id": "llm_234sdertfsdsfsdf",
                        "type": "retell-llm",
                        "version": 0,
                    },
                    "responsiveness": 1,
                    "ring_duration_ms": 30000,
                    "stt_mode": "fast",
                    "user_dtmf_options": {
                        "digit_limit": 1,
                        "termination_key": "#",
                        "timeout_ms": 1000,
                    },
                    "vocab_specialization": "general",
                    "voice_id": "11labs-Adrian",
                    "voice_model": "eleven_turbo_v2",
                    "voice_speed": 1,
                    "voice_temperature": 1,
                    "voicemail_option": {
                        "action": {
                            "text": "Please give us a callback tomorrow at 10am.",
                            "type": "static_text",
                        }
                    },
                    "volume": 1,
                    "webhook_timeout_ms": 10000,
                    "webhook_url": "https://webhook-url-here",
                },
                "conversation_flow": {
                    "begin_after_user_silence_ms": 2000,
                    "kb_config": {
                        "filter_score": 0.6,
                        "top_k": 3,
                    },
                    "knowledge_base_ids": ["kb_001", "kb_002"],
                    "model_choice": {
                        "model": "gpt-5",
                        "type": "cascading",
                        "high_priority": True,
                    },
                    "model_temperature": 0.7,
                    "start_speaker": "agent",
                    "tool_call_strict_mode": True,
                },
                "retell_llm": {
                    "begin_after_user_silence_ms": 2000,
                    "begin_message": "Hey I am a virtual assistant calling from Retell Hospital.",
                    "kb_config": {
                        "filter_score": 0.6,
                        "top_k": 3,
                    },
                    "knowledge_base_ids": ["string"],
                    "model": "gpt-4.1",
                    "model_high_priority": True,
                    "model_temperature": 0,
                    "s2s_model": "gpt-4o-realtime",
                    "start_speaker": "user",
                    "tool_call_strict_mode": True,
                },
            },
            agent_version=1,
            direction="inbound",
            from_number="+14157774444",
            metadata={},
            retell_llm_dynamic_variables={"customer_name": "bar"},
            to_number="+12137774445",
        )
        assert_matches_type(PhoneCallResponse, call, path=["response"])

    @parametrize
    def test_raw_response_register_phone_call(self, client: Retell) -> None:
        response = client.call.with_raw_response.register_phone_call(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call = response.parse()
        assert_matches_type(PhoneCallResponse, call, path=["response"])

    @parametrize
    def test_streaming_response_register_phone_call(self, client: Retell) -> None:
        with client.call.with_streaming_response.register_phone_call(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call = response.parse()
            assert_matches_type(PhoneCallResponse, call, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncCall:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncRetell) -> None:
        call = await async_client.call.retrieve(
            "119c3f8e47135a29e65947eeb34cf12d",
        )
        assert_matches_type(CallResponse, call, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncRetell) -> None:
        response = await async_client.call.with_raw_response.retrieve(
            "119c3f8e47135a29e65947eeb34cf12d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call = await response.parse()
        assert_matches_type(CallResponse, call, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncRetell) -> None:
        async with async_client.call.with_streaming_response.retrieve(
            "119c3f8e47135a29e65947eeb34cf12d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call = await response.parse()
            assert_matches_type(CallResponse, call, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `call_id` but received ''"):
            await async_client.call.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncRetell) -> None:
        call = await async_client.call.update(
            call_id="call_a4441234567890777c4a4a123e6",
        )
        assert_matches_type(CallResponse, call, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncRetell) -> None:
        call = await async_client.call.update(
            call_id="call_a4441234567890777c4a4a123e6",
            data_storage_setting="everything_except_pii",
            metadata={
                "customer_id": "cust_123",
                "notes": "Follow-up required",
            },
            override_dynamic_variables={"additional_discount": "15%"},
        )
        assert_matches_type(CallResponse, call, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncRetell) -> None:
        response = await async_client.call.with_raw_response.update(
            call_id="call_a4441234567890777c4a4a123e6",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call = await response.parse()
        assert_matches_type(CallResponse, call, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncRetell) -> None:
        async with async_client.call.with_streaming_response.update(
            call_id="call_a4441234567890777c4a4a123e6",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call = await response.parse()
            assert_matches_type(CallResponse, call, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `call_id` but received ''"):
            await async_client.call.with_raw_response.update(
                call_id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncRetell) -> None:
        call = await async_client.call.list()
        assert_matches_type(CallListResponse, call, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncRetell) -> None:
        call = await async_client.call.list(
            filter_criteria={
                "agent_id": ["agent_oBeDLoLOeuAbiuaMFXRtDOLriT12345"],
                "batch_call_id": ["string"],
                "call_status": ["ended"],
                "call_successful": [True],
                "call_type": ["phone_call"],
                "direction": ["inbound"],
                "disconnection_reason": ["user_hangup"],
                "duration_ms": {
                    "lower_threshold": 0,
                    "upper_threshold": 0,
                },
                "e2e_latency_p50": {
                    "lower_threshold": 0,
                    "upper_threshold": 0,
                },
                "from_number": ["string"],
                "in_voicemail": [True],
                "start_timestamp": {
                    "lower_threshold": 1738475411000,
                    "upper_threshold": 1738475421000,
                },
                "to_number": ["string"],
                "user_sentiment": ["Positive"],
                "version": [0],
            },
            limit=0,
            pagination_key="pagination_key",
            sort_order="ascending",
        )
        assert_matches_type(CallListResponse, call, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncRetell) -> None:
        response = await async_client.call.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call = await response.parse()
        assert_matches_type(CallListResponse, call, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncRetell) -> None:
        async with async_client.call.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call = await response.parse()
            assert_matches_type(CallListResponse, call, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncRetell) -> None:
        call = await async_client.call.delete(
            "119c3f8e47135a29e65947eeb34cf12d",
        )
        assert call is None

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncRetell) -> None:
        response = await async_client.call.with_raw_response.delete(
            "119c3f8e47135a29e65947eeb34cf12d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call = await response.parse()
        assert call is None

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncRetell) -> None:
        async with async_client.call.with_streaming_response.delete(
            "119c3f8e47135a29e65947eeb34cf12d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call = await response.parse()
            assert call is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `call_id` but received ''"):
            await async_client.call.with_raw_response.delete(
                "",
            )

    @parametrize
    async def test_method_create_phone_call(self, async_client: AsyncRetell) -> None:
        call = await async_client.call.create_phone_call(
            from_number="+14157774444",
            to_number="+12137774445",
        )
        assert_matches_type(PhoneCallResponse, call, path=["response"])

    @parametrize
    async def test_method_create_phone_call_with_all_params(self, async_client: AsyncRetell) -> None:
        call = await async_client.call.create_phone_call(
            from_number="+14157774444",
            to_number="+12137774445",
            agent_override={
                "agent": {
                    "agent_name": "Jarvis",
                    "allow_user_dtmf": True,
                    "ambient_sound": "coffee-shop",
                    "ambient_sound_volume": 1,
                    "backchannel_frequency": 0.9,
                    "backchannel_words": ["yeah", "uh-huh"],
                    "begin_message_delay_ms": 1000,
                    "boosted_keywords": ["retell", "kroger"],
                    "data_storage_setting": "everything",
                    "denoising_mode": "noise-cancellation",
                    "enable_backchannel": True,
                    "end_call_after_silence_ms": 600000,
                    "fallback_voice_ids": ["openai-Alloy", "deepgram-Angus"],
                    "interruption_sensitivity": 1,
                    "language": "en-US",
                    "max_call_duration_ms": 3600000,
                    "normalize_for_speech": True,
                    "opt_in_signed_url": True,
                    "pii_config": {
                        "categories": ["person_name"],
                        "mode": "post_call",
                    },
                    "post_call_analysis_data": [
                        {
                            "description": "The name of the customer.",
                            "name": "customer_name",
                            "type": "string",
                            "examples": ["John Doe", "Jane Smith"],
                        }
                    ],
                    "post_call_analysis_model": "gpt-4o-mini",
                    "pronunciation_dictionary": [
                        {
                            "alphabet": "ipa",
                            "phoneme": "ˈæktʃuəli",
                            "word": "actually",
                        }
                    ],
                    "reminder_max_count": 2,
                    "reminder_trigger_ms": 10000,
                    "response_engine": {
                        "llm_id": "llm_234sdertfsdsfsdf",
                        "type": "retell-llm",
                        "version": 0,
                    },
                    "responsiveness": 1,
                    "ring_duration_ms": 30000,
                    "stt_mode": "fast",
                    "user_dtmf_options": {
                        "digit_limit": 1,
                        "termination_key": "#",
                        "timeout_ms": 1000,
                    },
                    "vocab_specialization": "general",
                    "voice_id": "11labs-Adrian",
                    "voice_model": "eleven_turbo_v2",
                    "voice_speed": 1,
                    "voice_temperature": 1,
                    "voicemail_option": {
                        "action": {
                            "text": "Please give us a callback tomorrow at 10am.",
                            "type": "static_text",
                        }
                    },
                    "volume": 1,
                    "webhook_timeout_ms": 10000,
                    "webhook_url": "https://webhook-url-here",
                },
                "conversation_flow": {
                    "begin_after_user_silence_ms": 2000,
                    "kb_config": {
                        "filter_score": 0.6,
                        "top_k": 3,
                    },
                    "knowledge_base_ids": ["kb_001", "kb_002"],
                    "model_choice": {
                        "model": "gpt-5",
                        "type": "cascading",
                        "high_priority": True,
                    },
                    "model_temperature": 0.7,
                    "start_speaker": "agent",
                    "tool_call_strict_mode": True,
                },
                "retell_llm": {
                    "begin_after_user_silence_ms": 2000,
                    "begin_message": "Hey I am a virtual assistant calling from Retell Hospital.",
                    "kb_config": {
                        "filter_score": 0.6,
                        "top_k": 3,
                    },
                    "knowledge_base_ids": ["string"],
                    "model": "gpt-4.1",
                    "model_high_priority": True,
                    "model_temperature": 0,
                    "s2s_model": "gpt-4o-realtime",
                    "start_speaker": "user",
                    "tool_call_strict_mode": True,
                },
            },
            custom_sip_headers={"X-Custom-Header": "Custom Value"},
            ignore_e164_validation=True,
            metadata={},
            override_agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
            override_agent_version=1,
            retell_llm_dynamic_variables={"customer_name": "bar"},
        )
        assert_matches_type(PhoneCallResponse, call, path=["response"])

    @parametrize
    async def test_raw_response_create_phone_call(self, async_client: AsyncRetell) -> None:
        response = await async_client.call.with_raw_response.create_phone_call(
            from_number="+14157774444",
            to_number="+12137774445",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call = await response.parse()
        assert_matches_type(PhoneCallResponse, call, path=["response"])

    @parametrize
    async def test_streaming_response_create_phone_call(self, async_client: AsyncRetell) -> None:
        async with async_client.call.with_streaming_response.create_phone_call(
            from_number="+14157774444",
            to_number="+12137774445",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call = await response.parse()
            assert_matches_type(PhoneCallResponse, call, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_create_web_call(self, async_client: AsyncRetell) -> None:
        call = await async_client.call.create_web_call(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        )
        assert_matches_type(WebCallResponse, call, path=["response"])

    @parametrize
    async def test_method_create_web_call_with_all_params(self, async_client: AsyncRetell) -> None:
        call = await async_client.call.create_web_call(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
            agent_override={
                "agent": {
                    "agent_name": "Jarvis",
                    "allow_user_dtmf": True,
                    "ambient_sound": "coffee-shop",
                    "ambient_sound_volume": 1,
                    "backchannel_frequency": 0.9,
                    "backchannel_words": ["yeah", "uh-huh"],
                    "begin_message_delay_ms": 1000,
                    "boosted_keywords": ["retell", "kroger"],
                    "data_storage_setting": "everything",
                    "denoising_mode": "noise-cancellation",
                    "enable_backchannel": True,
                    "end_call_after_silence_ms": 600000,
                    "fallback_voice_ids": ["openai-Alloy", "deepgram-Angus"],
                    "interruption_sensitivity": 1,
                    "language": "en-US",
                    "max_call_duration_ms": 3600000,
                    "normalize_for_speech": True,
                    "opt_in_signed_url": True,
                    "pii_config": {
                        "categories": ["person_name"],
                        "mode": "post_call",
                    },
                    "post_call_analysis_data": [
                        {
                            "description": "The name of the customer.",
                            "name": "customer_name",
                            "type": "string",
                            "examples": ["John Doe", "Jane Smith"],
                        }
                    ],
                    "post_call_analysis_model": "gpt-4o-mini",
                    "pronunciation_dictionary": [
                        {
                            "alphabet": "ipa",
                            "phoneme": "ˈæktʃuəli",
                            "word": "actually",
                        }
                    ],
                    "reminder_max_count": 2,
                    "reminder_trigger_ms": 10000,
                    "response_engine": {
                        "llm_id": "llm_234sdertfsdsfsdf",
                        "type": "retell-llm",
                        "version": 0,
                    },
                    "responsiveness": 1,
                    "ring_duration_ms": 30000,
                    "stt_mode": "fast",
                    "user_dtmf_options": {
                        "digit_limit": 1,
                        "termination_key": "#",
                        "timeout_ms": 1000,
                    },
                    "vocab_specialization": "general",
                    "voice_id": "11labs-Adrian",
                    "voice_model": "eleven_turbo_v2",
                    "voice_speed": 1,
                    "voice_temperature": 1,
                    "voicemail_option": {
                        "action": {
                            "text": "Please give us a callback tomorrow at 10am.",
                            "type": "static_text",
                        }
                    },
                    "volume": 1,
                    "webhook_timeout_ms": 10000,
                    "webhook_url": "https://webhook-url-here",
                },
                "conversation_flow": {
                    "begin_after_user_silence_ms": 2000,
                    "kb_config": {
                        "filter_score": 0.6,
                        "top_k": 3,
                    },
                    "knowledge_base_ids": ["kb_001", "kb_002"],
                    "model_choice": {
                        "model": "gpt-5",
                        "type": "cascading",
                        "high_priority": True,
                    },
                    "model_temperature": 0.7,
                    "start_speaker": "agent",
                    "tool_call_strict_mode": True,
                },
                "retell_llm": {
                    "begin_after_user_silence_ms": 2000,
                    "begin_message": "Hey I am a virtual assistant calling from Retell Hospital.",
                    "kb_config": {
                        "filter_score": 0.6,
                        "top_k": 3,
                    },
                    "knowledge_base_ids": ["string"],
                    "model": "gpt-4.1",
                    "model_high_priority": True,
                    "model_temperature": 0,
                    "s2s_model": "gpt-4o-realtime",
                    "start_speaker": "user",
                    "tool_call_strict_mode": True,
                },
            },
            agent_version=1,
            metadata={},
            retell_llm_dynamic_variables={"customer_name": "bar"},
        )
        assert_matches_type(WebCallResponse, call, path=["response"])

    @parametrize
    async def test_raw_response_create_web_call(self, async_client: AsyncRetell) -> None:
        response = await async_client.call.with_raw_response.create_web_call(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call = await response.parse()
        assert_matches_type(WebCallResponse, call, path=["response"])

    @parametrize
    async def test_streaming_response_create_web_call(self, async_client: AsyncRetell) -> None:
        async with async_client.call.with_streaming_response.create_web_call(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call = await response.parse()
            assert_matches_type(WebCallResponse, call, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_register_phone_call(self, async_client: AsyncRetell) -> None:
        call = await async_client.call.register_phone_call(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        )
        assert_matches_type(PhoneCallResponse, call, path=["response"])

    @parametrize
    async def test_method_register_phone_call_with_all_params(self, async_client: AsyncRetell) -> None:
        call = await async_client.call.register_phone_call(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
            agent_override={
                "agent": {
                    "agent_name": "Jarvis",
                    "allow_user_dtmf": True,
                    "ambient_sound": "coffee-shop",
                    "ambient_sound_volume": 1,
                    "backchannel_frequency": 0.9,
                    "backchannel_words": ["yeah", "uh-huh"],
                    "begin_message_delay_ms": 1000,
                    "boosted_keywords": ["retell", "kroger"],
                    "data_storage_setting": "everything",
                    "denoising_mode": "noise-cancellation",
                    "enable_backchannel": True,
                    "end_call_after_silence_ms": 600000,
                    "fallback_voice_ids": ["openai-Alloy", "deepgram-Angus"],
                    "interruption_sensitivity": 1,
                    "language": "en-US",
                    "max_call_duration_ms": 3600000,
                    "normalize_for_speech": True,
                    "opt_in_signed_url": True,
                    "pii_config": {
                        "categories": ["person_name"],
                        "mode": "post_call",
                    },
                    "post_call_analysis_data": [
                        {
                            "description": "The name of the customer.",
                            "name": "customer_name",
                            "type": "string",
                            "examples": ["John Doe", "Jane Smith"],
                        }
                    ],
                    "post_call_analysis_model": "gpt-4o-mini",
                    "pronunciation_dictionary": [
                        {
                            "alphabet": "ipa",
                            "phoneme": "ˈæktʃuəli",
                            "word": "actually",
                        }
                    ],
                    "reminder_max_count": 2,
                    "reminder_trigger_ms": 10000,
                    "response_engine": {
                        "llm_id": "llm_234sdertfsdsfsdf",
                        "type": "retell-llm",
                        "version": 0,
                    },
                    "responsiveness": 1,
                    "ring_duration_ms": 30000,
                    "stt_mode": "fast",
                    "user_dtmf_options": {
                        "digit_limit": 1,
                        "termination_key": "#",
                        "timeout_ms": 1000,
                    },
                    "vocab_specialization": "general",
                    "voice_id": "11labs-Adrian",
                    "voice_model": "eleven_turbo_v2",
                    "voice_speed": 1,
                    "voice_temperature": 1,
                    "voicemail_option": {
                        "action": {
                            "text": "Please give us a callback tomorrow at 10am.",
                            "type": "static_text",
                        }
                    },
                    "volume": 1,
                    "webhook_timeout_ms": 10000,
                    "webhook_url": "https://webhook-url-here",
                },
                "conversation_flow": {
                    "begin_after_user_silence_ms": 2000,
                    "kb_config": {
                        "filter_score": 0.6,
                        "top_k": 3,
                    },
                    "knowledge_base_ids": ["kb_001", "kb_002"],
                    "model_choice": {
                        "model": "gpt-5",
                        "type": "cascading",
                        "high_priority": True,
                    },
                    "model_temperature": 0.7,
                    "start_speaker": "agent",
                    "tool_call_strict_mode": True,
                },
                "retell_llm": {
                    "begin_after_user_silence_ms": 2000,
                    "begin_message": "Hey I am a virtual assistant calling from Retell Hospital.",
                    "kb_config": {
                        "filter_score": 0.6,
                        "top_k": 3,
                    },
                    "knowledge_base_ids": ["string"],
                    "model": "gpt-4.1",
                    "model_high_priority": True,
                    "model_temperature": 0,
                    "s2s_model": "gpt-4o-realtime",
                    "start_speaker": "user",
                    "tool_call_strict_mode": True,
                },
            },
            agent_version=1,
            direction="inbound",
            from_number="+14157774444",
            metadata={},
            retell_llm_dynamic_variables={"customer_name": "bar"},
            to_number="+12137774445",
        )
        assert_matches_type(PhoneCallResponse, call, path=["response"])

    @parametrize
    async def test_raw_response_register_phone_call(self, async_client: AsyncRetell) -> None:
        response = await async_client.call.with_raw_response.register_phone_call(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call = await response.parse()
        assert_matches_type(PhoneCallResponse, call, path=["response"])

    @parametrize
    async def test_streaming_response_register_phone_call(self, async_client: AsyncRetell) -> None:
        async with async_client.call.with_streaming_response.register_phone_call(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call = await response.parse()
            assert_matches_type(PhoneCallResponse, call, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from retell import Retell, AsyncRetell
from tests.utils import assert_matches_type
from retell.types import (
    ChatResponse,
    ChatListResponse,
    ChatCreateChatCompletionResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestChat:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Retell) -> None:
        chat = client.chat.create(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        )
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Retell) -> None:
        chat = client.chat.create(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
            agent_version=1,
            metadata={},
            retell_llm_dynamic_variables={"customer_name": "bar"},
        )
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Retell) -> None:
        response = client.chat.with_raw_response.create(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = response.parse()
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Retell) -> None:
        with client.chat.with_streaming_response.create(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = response.parse()
            assert_matches_type(ChatResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Retell) -> None:
        chat = client.chat.retrieve(
            "16b980523634a6dc504898cda492e939",
        )
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Retell) -> None:
        response = client.chat.with_raw_response.retrieve(
            "16b980523634a6dc504898cda492e939",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = response.parse()
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Retell) -> None:
        with client.chat.with_streaming_response.retrieve(
            "16b980523634a6dc504898cda492e939",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = response.parse()
            assert_matches_type(ChatResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `chat_id` but received ''"):
            client.chat.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: Retell) -> None:
        chat = client.chat.update(
            chat_id="chat_98c1a2157aa0559144d67bb0729",
        )
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: Retell) -> None:
        chat = client.chat.update(
            chat_id="chat_98c1a2157aa0559144d67bb0729",
            data_storage_setting="everything",
            metadata={
                "customer_id": "cust_123",
                "notes": "Follow-up required",
            },
            override_dynamic_variables={"additional_discount": "15%"},
        )
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Retell) -> None:
        response = client.chat.with_raw_response.update(
            chat_id="chat_98c1a2157aa0559144d67bb0729",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = response.parse()
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Retell) -> None:
        with client.chat.with_streaming_response.update(
            chat_id="chat_98c1a2157aa0559144d67bb0729",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = response.parse()
            assert_matches_type(ChatResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `chat_id` but received ''"):
            client.chat.with_raw_response.update(
                chat_id="",
            )

    @parametrize
    def test_method_list(self, client: Retell) -> None:
        chat = client.chat.list()
        assert_matches_type(ChatListResponse, chat, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Retell) -> None:
        response = client.chat.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = response.parse()
        assert_matches_type(ChatListResponse, chat, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Retell) -> None:
        with client.chat.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = response.parse()
            assert_matches_type(ChatListResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_create_chat_completion(self, client: Retell) -> None:
        chat = client.chat.create_chat_completion(
            chat_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
            content="hi how are you doing?",
        )
        assert_matches_type(ChatCreateChatCompletionResponse, chat, path=["response"])

    @parametrize
    def test_raw_response_create_chat_completion(self, client: Retell) -> None:
        response = client.chat.with_raw_response.create_chat_completion(
            chat_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
            content="hi how are you doing?",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = response.parse()
        assert_matches_type(ChatCreateChatCompletionResponse, chat, path=["response"])

    @parametrize
    def test_streaming_response_create_chat_completion(self, client: Retell) -> None:
        with client.chat.with_streaming_response.create_chat_completion(
            chat_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
            content="hi how are you doing?",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = response.parse()
            assert_matches_type(ChatCreateChatCompletionResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_create_sms_chat(self, client: Retell) -> None:
        chat = client.chat.create_sms_chat(
            from_number="+12137771234",
            to_number="+14155551234",
        )
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    def test_method_create_sms_chat_with_all_params(self, client: Retell) -> None:
        chat = client.chat.create_sms_chat(
            from_number="+12137771234",
            to_number="+14155551234",
            metadata={},
            override_agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
            override_agent_version=1,
            retell_llm_dynamic_variables={"customer_name": "bar"},
        )
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    def test_raw_response_create_sms_chat(self, client: Retell) -> None:
        response = client.chat.with_raw_response.create_sms_chat(
            from_number="+12137771234",
            to_number="+14155551234",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = response.parse()
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    def test_streaming_response_create_sms_chat(self, client: Retell) -> None:
        with client.chat.with_streaming_response.create_sms_chat(
            from_number="+12137771234",
            to_number="+14155551234",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = response.parse()
            assert_matches_type(ChatResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_end(self, client: Retell) -> None:
        chat = client.chat.end(
            "16b980523634a6dc504898cda492e939",
        )
        assert chat is None

    @parametrize
    def test_raw_response_end(self, client: Retell) -> None:
        response = client.chat.with_raw_response.end(
            "16b980523634a6dc504898cda492e939",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = response.parse()
        assert chat is None

    @parametrize
    def test_streaming_response_end(self, client: Retell) -> None:
        with client.chat.with_streaming_response.end(
            "16b980523634a6dc504898cda492e939",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = response.parse()
            assert chat is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_end(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `chat_id` but received ''"):
            client.chat.with_raw_response.end(
                "",
            )


class TestAsyncChat:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncRetell) -> None:
        chat = await async_client.chat.create(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        )
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncRetell) -> None:
        chat = await async_client.chat.create(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
            agent_version=1,
            metadata={},
            retell_llm_dynamic_variables={"customer_name": "bar"},
        )
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncRetell) -> None:
        response = await async_client.chat.with_raw_response.create(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = await response.parse()
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncRetell) -> None:
        async with async_client.chat.with_streaming_response.create(
            agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = await response.parse()
            assert_matches_type(ChatResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncRetell) -> None:
        chat = await async_client.chat.retrieve(
            "16b980523634a6dc504898cda492e939",
        )
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncRetell) -> None:
        response = await async_client.chat.with_raw_response.retrieve(
            "16b980523634a6dc504898cda492e939",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = await response.parse()
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncRetell) -> None:
        async with async_client.chat.with_streaming_response.retrieve(
            "16b980523634a6dc504898cda492e939",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = await response.parse()
            assert_matches_type(ChatResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `chat_id` but received ''"):
            await async_client.chat.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncRetell) -> None:
        chat = await async_client.chat.update(
            chat_id="chat_98c1a2157aa0559144d67bb0729",
        )
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncRetell) -> None:
        chat = await async_client.chat.update(
            chat_id="chat_98c1a2157aa0559144d67bb0729",
            data_storage_setting="everything",
            metadata={
                "customer_id": "cust_123",
                "notes": "Follow-up required",
            },
            override_dynamic_variables={"additional_discount": "15%"},
        )
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncRetell) -> None:
        response = await async_client.chat.with_raw_response.update(
            chat_id="chat_98c1a2157aa0559144d67bb0729",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = await response.parse()
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncRetell) -> None:
        async with async_client.chat.with_streaming_response.update(
            chat_id="chat_98c1a2157aa0559144d67bb0729",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = await response.parse()
            assert_matches_type(ChatResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `chat_id` but received ''"):
            await async_client.chat.with_raw_response.update(
                chat_id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncRetell) -> None:
        chat = await async_client.chat.list()
        assert_matches_type(ChatListResponse, chat, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncRetell) -> None:
        response = await async_client.chat.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = await response.parse()
        assert_matches_type(ChatListResponse, chat, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncRetell) -> None:
        async with async_client.chat.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = await response.parse()
            assert_matches_type(ChatListResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_create_chat_completion(self, async_client: AsyncRetell) -> None:
        chat = await async_client.chat.create_chat_completion(
            chat_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
            content="hi how are you doing?",
        )
        assert_matches_type(ChatCreateChatCompletionResponse, chat, path=["response"])

    @parametrize
    async def test_raw_response_create_chat_completion(self, async_client: AsyncRetell) -> None:
        response = await async_client.chat.with_raw_response.create_chat_completion(
            chat_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
            content="hi how are you doing?",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = await response.parse()
        assert_matches_type(ChatCreateChatCompletionResponse, chat, path=["response"])

    @parametrize
    async def test_streaming_response_create_chat_completion(self, async_client: AsyncRetell) -> None:
        async with async_client.chat.with_streaming_response.create_chat_completion(
            chat_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
            content="hi how are you doing?",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = await response.parse()
            assert_matches_type(ChatCreateChatCompletionResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_create_sms_chat(self, async_client: AsyncRetell) -> None:
        chat = await async_client.chat.create_sms_chat(
            from_number="+12137771234",
            to_number="+14155551234",
        )
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    async def test_method_create_sms_chat_with_all_params(self, async_client: AsyncRetell) -> None:
        chat = await async_client.chat.create_sms_chat(
            from_number="+12137771234",
            to_number="+14155551234",
            metadata={},
            override_agent_id="oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
            override_agent_version=1,
            retell_llm_dynamic_variables={"customer_name": "bar"},
        )
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    async def test_raw_response_create_sms_chat(self, async_client: AsyncRetell) -> None:
        response = await async_client.chat.with_raw_response.create_sms_chat(
            from_number="+12137771234",
            to_number="+14155551234",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = await response.parse()
        assert_matches_type(ChatResponse, chat, path=["response"])

    @parametrize
    async def test_streaming_response_create_sms_chat(self, async_client: AsyncRetell) -> None:
        async with async_client.chat.with_streaming_response.create_sms_chat(
            from_number="+12137771234",
            to_number="+14155551234",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = await response.parse()
            assert_matches_type(ChatResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_end(self, async_client: AsyncRetell) -> None:
        chat = await async_client.chat.end(
            "16b980523634a6dc504898cda492e939",
        )
        assert chat is None

    @parametrize
    async def test_raw_response_end(self, async_client: AsyncRetell) -> None:
        response = await async_client.chat.with_raw_response.end(
            "16b980523634a6dc504898cda492e939",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = await response.parse()
        assert chat is None

    @parametrize
    async def test_streaming_response_end(self, async_client: AsyncRetell) -> None:
        async with async_client.chat.with_streaming_response.end(
            "16b980523634a6dc504898cda492e939",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = await response.parse()
            assert chat is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_end(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `chat_id` but received ''"):
            await async_client.chat.with_raw_response.end(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["PhoneNumberUpdateParams"]


class PhoneNumberUpdateParams(TypedDict, total=False):
    inbound_agent_id: Optional[str]
    """Unique id of agent to bind to the number.

    The number will automatically use the agent when receiving inbound calls. If set
    to null, this number would not accept inbound call.
    """

    inbound_agent_version: Optional[int]
    """Version of the inbound agent to bind to the number.

    If not provided, will default to latest version.
    """

    inbound_webhook_url: Optional[str]
    """
    If set, will send a webhook for inbound calls, where you can to override agent
    id, set dynamic variables and other fields specific to that call.
    """

    nickname: Optional[str]
    """Nickname of the number. This is for your reference only."""

    outbound_agent_id: Optional[str]
    """Unique id of agent to bind to the number.

    The number will automatically use the agent when conducting outbound calls. If
    set to null, this number would not be able to initiate outbound call without
    agent id override.
    """

    outbound_agent_version: Optional[int]
    """Version of the outbound agent to bind to the number.

    If not provided, will default to latest version.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .conversation_flow_response import ConversationFlowResponse

__all__ = ["ConversationFlowListResponse"]

ConversationFlowListResponse: TypeAlias = List[ConversationFlowResponse]



import json

from retell.lib.webhook_auth import symmetric  # type: ignore


def test_symmetric_signature() -> None:
    post_data = {
        "name": "some_function_name",
        "args": {"some_arg": "ABC123"},
    }
    body = json.dumps(post_data, separators=(",", ":"), ensure_ascii=False)
    api_key = "fake-api-key"

    signature = symmetric["sign"](body, api_key)  # type: ignore
    assert symmetric["verify"](body, api_key, signature)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from .._types import FileTypes, SequenceNotStr

__all__ = ["KnowledgeBaseAddSourcesParams", "KnowledgeBaseText"]


class KnowledgeBaseAddSourcesParams(TypedDict, total=False):
    knowledge_base_files: SequenceNotStr[FileTypes]
    """Files to add to the knowledge base.

    Limit to 25 files, where each file is limited to 50MB.
    """

    knowledge_base_texts: Iterable[KnowledgeBaseText]
    """Texts to add to the knowledge base."""

    knowledge_base_urls: SequenceNotStr[str]
    """URLs to be scraped and added to the knowledge base. Must be valid urls."""


class KnowledgeBaseText(TypedDict, total=False):
    text: Required[str]
    """Text to add to the knowledge base."""

    title: Required[str]
    """Title of the text."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, TypedDict

__all__ = ["ChatCreateSMSChatParams"]


class ChatCreateSMSChatParams(TypedDict, total=False):
    from_number: Required[str]
    """The phone number to send SMS from in E.164 format.

    Must be a number purchased from Retell or imported to Retell with SMS
    capability.
    """

    to_number: Required[str]
    """The phone number to send SMS to in E.164 format"""

    metadata: object
    """An arbitrary object for storage purpose only.

    You can put anything here like your internal customer id associated with the
    chat. Not used for processing. You can later get this field from the chat
    object.
    """

    override_agent_id: str
    """For this particular chat, override the agent used with this agent id.

    This does not bind the agent to this number, this is for one time override.
    """

    override_agent_version: int
    """For this particular chat, override the agent version used with this version.

    This does not bind the agent version to this number, this is for one time
    override.
    """

    retell_llm_dynamic_variables: Dict[str, object]
    """
    Add optional dynamic variables in key value pairs of string that injects into
    your Response Engine prompt and tool description. Only applicable for Response
    Engine.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["McpToolDefinition"]


class McpToolDefinition(BaseModel):
    description: str
    """Description of what the MCP tool does."""

    input_schema: Dict[str, object] = FieldInfo(alias="inputSchema")
    """JSON schema defining the input parameters for the tool."""

    name: str
    """Name of the MCP tool."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel

__all__ = [
    "BatchTestResponse",
    "ResponseEngine",
    "ResponseEngineResponseEngineRetellLm",
    "ResponseEngineResponseEngineCustomLm",
    "ResponseEngineResponseEngineConversationFlow",
]


class ResponseEngineResponseEngineRetellLm(BaseModel):
    llm_id: str
    """id of the Retell LLM Response Engine."""

    type: Literal["retell-llm"]
    """type of the Response Engine."""

    version: Optional[float] = None
    """Version of the Retell LLM Response Engine."""


class ResponseEngineResponseEngineCustomLm(BaseModel):
    llm_websocket_url: str
    """LLM websocket url of the custom LLM."""

    type: Literal["custom-llm"]
    """type of the Response Engine."""


class ResponseEngineResponseEngineConversationFlow(BaseModel):
    conversation_flow_id: str
    """ID of the Conversation Flow Response Engine."""

    type: Literal["conversation-flow"]
    """type of the Response Engine."""

    version: Optional[float] = None
    """Version of the Conversation Flow Response Engine."""


ResponseEngine: TypeAlias = Annotated[
    Union[
        ResponseEngineResponseEngineRetellLm,
        ResponseEngineResponseEngineCustomLm,
        ResponseEngineResponseEngineConversationFlow,
    ],
    PropertyInfo(discriminator="type"),
]


class BatchTestResponse(BaseModel):
    creation_timestamp: int
    """Timestamp when the batch job was created (milliseconds since epoch)"""

    error_count: int
    """Number of test cases that encountered errors"""

    fail_count: int
    """Number of test cases that failed"""

    pass_count: int
    """Number of test cases that passed"""

    response_engine: ResponseEngine

    status: Literal["in_progress", "complete"]
    """Status of the batch job"""

    test_case_batch_job_id: str
    """Unique identifier for the test case batch job"""

    total_count: int
    """Total number of test cases in the batch"""

    user_modified_timestamp: int
    """Timestamp when the batch job was last modified (milliseconds since epoch)"""
